"""User-defined exceptions.
"""
__copyright__ = "Copyright (C) 2021  Matt Ferreira"
__license__ = "Apache License"


class Error(Exception):
    """Base class for other exceptions"""
    pass


class APIConnectionError(Error):
    """Raised when connection to API breaks"""
    pass


class MissingConfiguration(Error):
    """Missing variables in configuration file"""
    pass


class InputError(Error):
    """Incorrect input provided to function or class"""
    pass



"""Functions to manipulate network IPs.
"""
__copyright__ = "Copyright (C) 2020-2021  Matt Ferreira"
__license__ = "Apache License"

from netaddr import IPNetwork


def get_all_network_ips(network):
    """Given a network (0.0.0.0/24) return all possible ip addresses

    arg:
        network (str): IP network to extract IP's from

    return (list): IP's in given network
    """
    return [str(ip) for ip in IPNetwork(network)]


def get_all_networks_ips(networks):
    """Given a list of networks return all possible ip addresses.

    arg:
        networks (list): List of IP networks to extract IP's from

    return (list): [ ['IP', 'Network'], ['IP2', 'Network'] ]
    """
    final_list = []
    for network in networks:
        ips = get_all_network_ips(network)
        final = [[ip, network] for ip in ips]
        final_list = final_list + final
    return final_list



"""Classes and functions for communicating with Jira.
"""
__copyright__ = "Copyright (C) 2020-2021  Matt Ferreira"
__license__ = "Apache License"

import logging
from jira import JIRA
from getpass import getpass

from avmp.core.exceptions import InputError


class JiraToolsAPI:
    def __init__(self, jira_server_link, username=None, password=None):
        """Initalizes the Jira API connector. If a username or password is not provided you will be prompted for it.

        args:
            jira_server_link (str): Link to the Jira server to touch API

        kwargs:
            username (str): Overwrites jira username prompt
            password (str): Overwrites jira password prompt

        return: None
        """
        self.jira_server_link = jira_server_link
        self.jira_options = {"server": self.jira_server_link}

        if username == None:
            username = input("Username: ")
        if password == None:
            password = getpass()

        self.username = username
        self.password = password

        self._JIRA = JIRA(self.jira_options, basic_auth=(
            self.username, self.password))
        logging.info(
            f"Authenticated successfully with Jira with {self.username}")

    def create(self, data):
        """Create a single Jira ticket.

        args:
            data (dict): Fields required or needed to create the ticket.

        return (str): Ticket number / 'False' if fails
        """
        try:
            jira_ticket = self._JIRA.create_issue(fields=data)
            logging.info(
                f"Successfully created Jira issue '{jira_ticket.key}'")
            return jira_ticket.key

        except Exception as error:
            logging.debug(
                f"Failed to create Jira issue '{jira_ticket.key}'\n\n{error}\n\n")
            return False

    def link(self, issue_from, issue_to, issue_link_name=None):
        """Link two issues together. Defaults to 'Relates' unless issue_link_name is specified.

        args:
            issue_from (str): Issue that will be linked from.
            issue_to (str): Issue that will be linked to.


        kwargs:
            issue_link_name (str): issue link name that should be applied.

        return (bool): Will return 'True' if it completed successfully.
        """
        try:
            self._JIRA.create_issue_link(issue_link_name, issue_from, issue_to)
            logging.info(
                f"Successfully created a '{issue_link_name}' link between '{issue_from}' and '{issue_to}'.")
            return True

        except Exception as error:
            logging.debug(
                f"Failed to create a link between '{issue_from}' and '{issue_to}'\n\n{error}\n\n")
            return False

    def label(self, issue, labels):
        """Apply labels to a given issue.

        args:
            issue (str): Issue that labels will be applied to.
            labels (list): list of labels that should be applied to the issue.

        Return (bool): Will return 'True' if it completed successfully.
        """
        if type(labels) == list:
            try:
                issue_instance = self._JIRA.issue(issue)
                issue_instance.update(
                    fields={"labels": issue_instance.fields.labels + labels})
                logging.info(
                    f"Successfully added labels '{labels}' to '{issue}'")
                return True

            except Exception as error:
                logging.debug(
                    f"Failed to add labels '{labels}' to '{issue}'\n\n{error}\n\n")
                return False

        else:
            raise InputError('A list must be passed to the labels argument')

    def comment(self, issue, comment):
        """Apply a comment to a given issue.

         args:
             issue (str): Issue that comment will be applied to.
             comment (str): comment that should be applied to the issue.

         return (bool): Will return 'True' if it completed successfully.
         """
        try:
            self._JIRA.add_comment(issue, comment)
            logging.info(
                f"Successfully added comment '{comment}' to '{issue}'")
            return True
        except Exception as error:
            logging.debug(
                f"Failed to add comment '{comment}' to '{issue}'\n\n{error}\n\n")
            return False

    def log_work(self, issue, time_spent, comment=None):
        """Log work to a given issue.

        args:
            issue (str): Issue to log work.
            time_spent (str): Time that should be logged to the issue.

        kwargs:
            comment (str): Description of what this time represents.

        return (bool): Will return 'True' if it completed successfully.
        """
        try:
            if comment != None and type(comment) == str:
                self._JIRA.add_worklog(issue, time_spent, comment=comment)
            else:
                self._JIRA.add_worklog(issue, time_spent)
            logging.info(f"Successfully logged time to '{issue}'")
            return True

        except Exception as error:
            logging.info(
                f"Failed to log work to '{issue}' See debug logs for more.")
            logging.debug(f"\n{error}\n")
            return False

    def add_attachment(self, issue, attachment):
        """Attach file to Jira issue.

        args:
            issue (str): Issue name
            attachment (str): Location of file that should be attached.

        Return (bool): Will return 'True' if completed successfully
        """
        assert isinstance(issue, str)
        assert isinstance(attachment, str)

        try:
            self._JIRA.add_attachment(issue=issue, attachment=attachment)
            logging.info(f'Successfully attached document to "{issue}"')
            return True

        except Exception as error:
            logging.debug(
                f"Failed to attach document to '{issue}'\n\n{error}\n\n")
            return False

    def update_status(self, id, end_status, transfer_statuses=[], timeout_attempts=10):
        """Change issue to desired status.

        Due to the workflow features of Jira it might not be possible to transition
        directly to the wanted status, intermediary statuses might be required and
        this funcation allows for that using 'transfer_statuses'.

        args:
            id (str): Issue id for status update
            end_status (str): Name of status to update ticket to.

        kwargs:
            transfer_statuses (list): Ordered list of intermediary statuses
            timeout_attempts (num): Number of times before while loop times out.

        return (bool): Will return 'True' if completed successfully
        """
        while timeout_attempts != 0:
            transitions = self._JIRA.transitions(id)
            for transition in transitions:
                if transition['name'] == end_status:
                    jira_ticket = self._JIRA.transition_issue(
                        id, transition['id'])
                    logging.info(
                        f"Updated status of '{id}' to '{end_status}'")
                    return True
                elif transition['name'] in transfer_statuses:
                    jira_ticket = self._JIRA.transition_issue(
                        id, transition['id'])
            timeout_attempts -= 1
        logging.debug(
            f"Failed to update status of '{id}' to end_status ({end_status})")
        return False



"""Runtime for AVMP.
"""
__copyright__ = "Copyright (C) 2021  Matt Ferreira"
__license__ = "Apache License"

import os
import socket
import logging
from datetime import datetime, timedelta

from avmp.core.models import App
from avmp.core.exceptions import MissingConfiguration
from avmp.utils.logging_utils import logging_setup
from avmp.tools.tenable_tools import (TenableToolsCSV,
                                      build_jira_description)
from avmp.utils.vuln_db import TenableSqliteVulnDB


def main(config, process_config):
    os.system('cls' if os.name == 'nt' else 'clear')
    logging_setup(os.path.basename(__file__), stdout=True)

    app = App(config, process_config)
    app.jiraAPIcon()

    # Check for process_type variable in scan config
    if "process_type" in app.process_config:
        # Determine scan type and run function for said type
        if 'static' in app.process_config['process_type']:
            static(app)
        elif 'dynamic' in app.process_config['process_type']:
            dynamic(app)
        else:
            message = '"process_type" (static or dynamic) is required. See documentation for examples.'
            logging.error(message)
            raise MissingConfiguration(message)
    else:
        message = 'No "process_type" variable provided in scan config. See documentation for examples.'
        logging.error(message)
        raise MissingConfiguration(message)


def static(app):
    """Generate static tickets found in config file.
    """
    # Create parent ticket
    logging.info('Creating tickets...')
    parent_ticket = app.jiraAPI.create(app.process_config['parent_ticket'])
    logging.info('{} used "{}" to create parent ticket "{}" successfully in Jira'.format(
        socket.gethostname(), app.config['creds']['jira']['username'], parent_ticket))

    # Log time saved
    jira_log_time(app, parent_ticket)

    logging.info(
        'Successfully created parent ticket ({})'.format(parent_ticket))

    # Create sub-tasks
    for key in app.process_config['sub_tasks']:

        # Set parent ticket in config if it doesn't exist
        if "parent" not in app.process_config['sub_tasks'][key]:
            app.process_config['sub_tasks'][key]['parent'] = {
                "key": parent_ticket}

        child = app.jiraAPI.create(app.process_config['sub_tasks'][key])
        logging.info('{} used "{}" to create child ticket "{}" under "{}" successfully in Jira'.format(
            socket.gethostname(), app.config['creds']['jira']['username'], child, parent_ticket))

        # Log time saved
        jira_log_time(app, child)

        logging.info('Successfully created sub_task ticket ({})'.format(child))

    logging.info('Tickets created successfully')


def dynamic(app):
    """Generate tickets for Tenable vulnerabilities.
    """
    # Setup connection to Tenable IO
    app.tenAPIcon()

    # Get raw scan data
    logging.info('Checking Tenable for new scan.')

    if app.tenAPI.check_scan_in_progress(app.process_config['scan_name']) == True:
        logging.info('Latest scan is still running, try again later.')
        return False
    else:
        filepath = app.tenAPI.export_latest_scan(app.process_config['scan_name'],
                                                 os.path.join(
            os.getcwd(), 'data', 'scans'),
            overwrite=False)

        # Build vulnerability database
        db = TenableSqliteVulnDB(app.process_config['ticket_db_filepath'])

        logging.info('Starting scan data import')
        items = TenableToolsCSV(
            filepath, min_cvss_score=app.process_config['min_cvss_score']).group_by('Plugin ID')

        if items == None:
            logging.debug('No data was found given the min_cvss_score')
            return False

        tickets = TenableToolsCSV.organize(items)
        logging.info('Completed scan data import')
        logging.info('Creating tickets for "{}"'.format(
            app.process_config['scan_name']))

        ticket_counter = 0
        for ticket in tickets.values():
            # Looping according to the max ticket count
            if ticket_counter < app.process_config['max_tickets'] or app.process_config['max_tickets'] == 0:

                data = {**app.process_config['data']}

                # Check to ensure all required fields are included
                if len(app.config['types']) > 0 and data['project']['key'] in app.config['types']:
                    missing_fields = []
                    for field in app.config['types'][data['project']['key']]:
                        if field not in app.process_config['data']:
                            missing_fields.append(field)
                    if len(missing_fields) != 0:
                        raise NameError(
                            f"The following fields were missing {missing_fields}")

                # Append variable data to data fields
                data['summary'] += ticket['Vuln Data']['Synopsis']
                data['summary'] = data['summary'].replace('\n', ' ')
                data['description'] += build_jira_description(ticket)
                if data['priority']['id'] == '':
                    # Selects proper priority rating inside of Jira
                    if ticket['Vuln Data']['Risk'] == 'Critical':
                        data['priority']['id'] = app.config['priorities']['Critical']
                    elif ticket['Vuln Data']['Risk'] == 'High':
                        data['priority']['id'] = app.config['priorities']['High']
                    elif ticket['Vuln Data']['Risk'] == 'Medium':
                        data['priority']['id'] = app.config['priorities']['Medium']
                    else:
                        data['priority']['id'] = app.config['priorities']['Low']

                if data['duedate'] == '':
                    # Build due date
                    today = datetime.now()
                    if ticket['Vuln Data']['Risk'] in app.config['due_dates']:
                        plus_days = app.config['due_dates'][ticket['Vuln Data']['Risk']]
                    else:
                        plus_days = app.config['due_dates']['Low']

                    final_date = today + timedelta(days=int(plus_days))
                    data['duedate'] = final_date.strftime("%Y-%m-%d")

                ip_list = set([x['IP Address']
                               for x in ticket['Hosts'].values()])

                # Check for already open tickets
                dups = db.get_all_tickets_by_plugin_id(
                    ticket['Vuln Data']['Plugin ID'])

                # TODO: Add abilty to open ticket for new IP's and link to existing vuln ticket.

                if len(dups) == 0 or app.process_config['allow_ticket_duplication'] == True:
                    if len(dups) != 0:
                        logging.info('Plugin ID ({}) has an open ticket. Creating duplicate...'.format(
                            ticket['Vuln Data']['Plugin ID']))

                    current = app.jiraAPI.create(data)
                    ticket_counter += 1
                    db.add_ticket(current, ticket['Vuln Data']
                                  ['Plugin ID'], app.process_config['default_ticket_status'], list(ip_list))

                    # Attempt adding comment(s) to ticket
                    if app.process_config['comments'] != '':
                        for comment in app.process_config['comments']:
                            try:
                                app.jiraAPI.comment(current, comment)
                                logging.info(
                                    'Successfully applied comment on {}'.format(current))
                            except:
                                logging.error(
                                    'Failed to apply comment on {}'.format(current))

                    # Log time saved
                    jira_log_time(app, current)

                    # Link ticket back to root ticket
                    if app.process_config['root_ticket'] != "":
                        try:
                            app.jiraAPI.link(app.process_config['root_ticket'], current,
                                             issue_link_name='depends on')
                            logging.info('Linked {} to {}'.format(
                                app.process_config['root_ticket'], current))
                        except:
                            logging.error('Failed to link {} to root ticket {}'.format(
                                current, app.process_config['root_ticket']))
                else:
                    logging.info('Plugin ID ({}) has an open ticket. Skipping...'.format(
                        ticket['Vuln Data']['Plugin ID']))


def jira_log_time(app, ticket):
    """Log time to newly created ticket, if fails log against root ticket if exists.

    app (class): Instance of the current runtime
    ticket (str): Ticket that was just created

    return (bool): Confirmation if time was logged.
    """
    logged_work = app.jiraAPI.log_work(
        ticket, app.process_config['time_saved_per_ticket'], comment=app.process_config['time_saved_comment'])

    if "root_ticket" in app.process_config and app.process_config['root_ticket'] != "" and logged_work == False:
        root_comment = '{} - {}'.format(
            ticket, app.process_config['time_saved_comment'])
        root_ticket_logged_work = app.jiraAPI.log_work(
            app.process_config['root_ticket'], app.process_config['time_saved_per_ticket'], comment=root_comment)

    if logged_work == True or root_ticket_logged_work == True:
        return True
    else:
        return False


if __name__ == '__main__':
    import json
    config_location = 'avmp/test_data/config.json'
    config = json.load(open(config_location, 'r'))

    config_location = 'avmp/test_data/process_configs/asv_process_config.json'
    process_config = json.load(open(config_location, 'r'))

    main(config, process_config)



"""Python tools for package.
"""
__copyright__ = "Copyright (C) 2020-2021  Matt Ferreira"
__license__ = "Apache License"

import os
import json
import logging


class DataSetup:
    def __init__(self, filename):
        """Standard data file setup.

        args:
            filename (str): Script filename.

        Return: None
        """
        assert isinstance(filename, str)
        self.filename = filename[:-3]+".json"
        self.filepath = 'data/' + self.filename

        # Check/Create data folder
        if os.path.isdir('data') == False:
            os.mkdir('data')
            logging.info('Creating data folder.')
        if not os.path.isfile(self.filepath):
            with open(self.filepath, 'w') as openFile:
                openFile.write('{}')

    def get_data(self):
        """Pull data from file

        Return (dict): Data found in data file.
        """
        with open(self.filepath, 'r') as openFile:
            data = json.load(openFile)
            logging.info('Loaded JSON data file.')
        return data

    def put_data(self, data):
        """Re-write data to file

        args:
            data (dict): Data to be converted to json and written to file

        Return (bool): If successful True, else False
        """
        assert isinstance(data, dict)
        try:
            with open(self.filepath, 'w') as openFile:
                openFile.write(json.dumps(data))

            logging.info('JSON exported to data file.')

            return True
        except Exception as e:
            logging.warning(e)
            return False



"""
"""
__copyright__ = "Copyright (C) 2021  Matt Ferreira"
__license__ = "Apache License"

import logging

from avmp.tools.jira_tools import JiraToolsAPI
from avmp.utils.vuln_db import TenableSqliteVulnDB


def main(vuln_db_filepath, config):
    root_ticket = input('Root ticket for logging time: ')
    db = TenableSqliteVulnDB(vuln_db_filepath)
    tickets = db.get_all_tickets()

    try:
        jiraAPI = JiraToolsAPI(config['creds']['jira']['server'],
                               username=config['creds']['jira']['username'],
                               password=config['creds']['jira']['password'])
    except Exception as e:
        logging.debug(
            f"{config['creds']['jira']['api_username']} failed to authenticate with Jira.\n\n{e}\n\n")

    COMMENT = 'Updating vulnerability database.'

    for num, ticket in enumerate(tickets):
        print(f'Updating {ticket[0]} ({num+1} of {len(tickets)})')
        data = jiraAPI._JIRA.issue(ticket[0])
        logged_work = jiraAPI.log_work(
            ticket[0], '5m', comment=COMMENT)

        # Attempt logging time to root ticket if original ticket does not allow
        if root_ticket != "" and logged_work == False:
            root_comment = '{} - {}'.format(
                ticket[0], COMMENT)
            jiraAPI.log_work(
                root_ticket, '5m', comment=root_comment)

        db.update_status_by_ticket_number(ticket[0], str(data.fields.status))



"""Class functions for managing a sqlite database for Tenable vulnerabilities.
"""
__copyright__ = "Copyright (C) 2020-2021  Matt Ferreira"
__license__ = "Apache License"

import os
import sqlite3
import datetime
import logging


class TenableSqliteVulnDB:
    def __init__(self, filepath, ignore_statuses=['Done', 'Closed', 'Platform Complete']):
        isinstance(filepath, str)
        isinstance(ignore_statuses, list)

        self.filepath = filepath
        self.db_name = os.path.basename(filepath)

        if not os.path.exists(filepath):
            logging.info('DB Not Found...')
            self._build_db()

        self.con = sqlite3.connect(filepath)
        self.ignore_statuses = ignore_statuses

    def _build_db(self):
        logging.info('Building DB now...')

        con = sqlite3.connect(self.db_name)

        con.execute("""
                CREATE TABLE "tickets" (
                "ticket_id"	TEXT NOT NULL,
                "plugin_id"	TEXT NOT NULL,
                "status"	INTEGER,
                "created_date"	TEXT NOT NULL,
                "modified_date"	TEXT NOT NULL,
                PRIMARY KEY("ticket_id"))
                """)
        con.execute("""
                CREATE TABLE "hosts" (
	            "host"	TEXT NOT NULL,
                PRIMARY KEY("host"))
                """)
        con.execute("""
                CREATE TABLE "hosts_tickets" (
	            "id"	INTEGER NOT NULL UNIQUE,
	            "host"	INTEGER NOT NULL,
                "ticket_id"	INTEGER NOT NULL,
                FOREIGN KEY("host") REFERENCES "hosts"("host"),
                FOREIGN KEY("ticket_id") REFERENCES "hosts"("ticket_id"),
                PRIMARY KEY("id" AUTOINCREMENT))
                """)
        con.close()

    def add_ticket(self, ticket_number, plugin_id, status, hosts, date=datetime.datetime.now().strftime('%Y-%m-%d')):
        """Add a created ticket to database.

        args:
            ticket_number (str): Identification value for ticketing system.
            plugin_id (str): Plugin ID provided by Tenable.
            status (str): Status of new ticket in ticketing system.
            hosts (list): List of ip's associated with ticket.
        kwargs:
            date (str): [default: current date YYYY-MM-DD] Date ticket was created.

        Return (bool): Confirmation of completion.
        """
        isinstance(ticket_number, str)
        isinstance(plugin_id, str)
        isinstance(status, str)
        isinstance(hosts, list)
        isinstance(date, str)

        try:
            assert self.check_by_ticket_number(
                ticket_number) == False, 'Ticket already exists in DB'
        except AssertionError:
            logging.exception('"{}" is already in {}'.format(
                ticket_number, self.db_name))
            return False

        sql = 'INSERT INTO tickets (ticket_id, plugin_id, status, created_date, modified_date) '
        sql += 'values ("{}","{}","{}","{}","{}")'.format(ticket_number,
                                                          plugin_id,
                                                          status,
                                                          date,
                                                          date
                                                          )
        with self.con:
            self.con.execute(sql)

        for ip in hosts:
            if not self.check_by_host(ip):
                logging.info(f"Adding {ip} to db")
                sql = 'INSERT INTO hosts (host)'
                sql += 'values ("{}")'.format(ip)

                with self.con:
                    self.con.execute(sql)

            sql = 'INSERT INTO hosts_tickets (host, ticket_id)'
            sql += 'values ("{}","{}")'.format(ip, ticket_number)

            with self.con:
                self.con.execute(sql)

        return True

    def check_by_host(self, host):
        """Given a host check if it exists in the database.

        args:
            host (str): IP/Hostname to check database for

        return (bool): Confirmation of existance.
        """
        assert isinstance(host, str)
        sql = f'SELECT * FROM hosts WHERE host="{host}"'
        with self.con:
            data = self.con.execute(sql).fetchone()
        if data == None:
            logging.info(f'"{host}" does not exist.')
            return False
        else:
            logging.info(f'"{host}" exists.')
            return True

    def check_by_ticket_number(self, ticket_number):
        """Given a ticket number check if it exists in the database.

        args:
            ticket_number (str): Identification value for ticketing system to check.

        return (bool): Confirmation of existance.
        """
        assert isinstance(ticket_number, str)
        sql = f'SELECT * FROM tickets WHERE ticket_id="{ticket_number}"'
        with self.con:
            data = self.con.execute(sql).fetchone()
        if data == None:
            logging.info(f'"{ticket_number}" does not exist.')
            return False
        else:
            logging.info(f'"{ticket_number}" exists.')
            return True

    def update_status_by_ticket_number(self, ticket_number, status, date=datetime.datetime.now().strftime('%Y-%m-%d')):
        """Update status on database row of given ticket_number.

        args:
            ticket_number (str): Identification value for ticketing system to update.
            status (str): New status to be added to database row.
        kwargs:
            date (str): [default: current date YYYY-MM-DD] Date ticket was created.

        Return (bool): Confirmation of update.
        """
        isinstance(ticket_number, str)
        isinstance(status, str)
        isinstance(date, str)

        sql = f'UPDATE tickets SET status="{status}", modified_date="{date}" WHERE ticket_id="{ticket_number}"'

        try:
            with self.con:
                data = self.con.execute(sql)
                logging.info(
                    f'Successfully updated status for "{ticket_number}"')
                return True
        except:
            logging.info(
                f'"{ticket_number}" is either already in a completed state or does not exist in "{self.db_name}"')
            return False

    def check_by_plugin_id(self, plugin_id):
        """Given a plugin id check if it exists in the database

        args:
            plugin_id (str): Plugin id for a given vulnerability.

        return (bool): Confirmation of exisitance.
        """
        assert isinstance(plugin_id, str)
        sql = f'SELECT * FROM tickets WHERE plugin_id="{plugin_id}"'
        with self.con:
            data = self.con.execute(sql).fetchall()
        if data == None:
            logging.info(f'"{plugin_id}" does not exist.')
            return False
        else:
            logging.info(f'"{plugin_id}" exists.')
            return True

    def get_by_plugin_id(self, plugin_id):
        """Given a plugin id return ticket_id and status

        args:
            plugin_id (str): Plugin id for a given vulnerability.

        return (list): List of each ticket_id and status
        """
        assert isinstance(plugin_id, str)
        sql = f'SELECT * FROM tickets WHERE plugin_id="{plugin_id}"'
        with self.con:
            data = self.con.execute(sql).fetchall()
        if data == None:
            logging.info((f'No data found for "{plugin_id}"'))
            return None
        else:
            logging.info(f'""')

        # TODO: This is still incomplete and is a work in progress

    def get_all_tickets_by_plugin_id(self, plugin_id):
        """Given a plugin id get all tickets with working statuses.
            i.e. Ticket status can not be in self.ignore_statuses

        args:
            plugin_id (str): Plugin if for a given vulnerability

        return (list): List of all tickets.
        """
        assert isinstance(plugin_id, str)
        ignore_statuses = ['status != "{}"'.format(
            x) for x in self.ignore_statuses]
        where = ' AND '.join(ignore_statuses)
        sql = f'SELECT * FROM tickets WHERE plugin_id="{plugin_id}" AND {where}'

        with self.con:
            data = self.con.execute(sql).fetchall()

        return data

    def get_all_tickets(self):
        if len(self.ignore_statuses) != 0:
            ignore_statuses = ['status != "{}"'.format(
                x) for x in self.ignore_statuses]
            where = ' AND '.join(ignore_statuses)
            sql = f'SELECT ticket_id FROM tickets WHERE {where}'
        else:
            sql = 'SELECT ticket_id FROM tickets'

        with self.con:
            data = self.con.execute(sql).fetchall()

        return data


if __name__ == '__main__':
    db = TenableSqliteVulnDB('tickets.db')
    print(db.get_all_tickets_by_plugin_id('5192'))



"""
      __      ____  __ _____  
     /\ \    / /  \/  |  __ \ 
    /  \ \  / /| \  / | |__) |
   / /\ \ \/ / | |\/| |  ___/ 
  / ____ \  /  | |  | | |     
 /_/    \_\/   |_|  |_|_|     
                              
A collection of tools for managing and automating vulnerability management.

Usage:
    avmp run [--config filepath] <process_config>
    avmp update [--config filepath] <ticket_db_filepath> 
    avmp -h | --help
    avmp --version

Options:
    -h --help           Show this screen.
    --version           Show version.
    --config=filepath   AVMP configuration file [default: config.json]
"""
__copyright__ = "Copyright (C) 2020-2021  Matt Ferreira"
__license__ = "Apache License"

from docopt import docopt
import json

from avmp.core import wrapper, updater


def main():
    args = docopt(__doc__, version='0.0.2')

    if args['run'] == True:

        with open(args['--config'], 'r') as openFile:
            config = json.load(openFile)
        with open(args['<process_config>'], 'r') as openFile:
            process_config = json.load(openFile)

        wrapper.main(config, process_config)

    elif args['update'] == True:

        with open(args['--config'], 'r') as openFile:
            config = json.load(openFile)

        updater.main(args['<ticket_db_filepath>'], config)



"""Class models for AVMP.
"""
__copyright__ = "Copyright (C) 2021  Matt Ferreira"
__license__ = "Apache License"

import os
import logging

from avmp.tools.tenable_tools import TenableToolsAPI
from avmp.tools.jira_tools import JiraToolsAPI
from avmp.core.exceptions import MissingConfiguration


class App():
    """Data associated with running AVMP.
    """

    def __init__(self, config, process_config):
        """Build AVMP app given the config file.

        args:
            config (dict): Imported json data from config
            process_config (dict): Imported json data from process_config. 
                                    None is also accepted.

        return: None
        """
        assert isinstance(config, dict)
        assert isinstance(process_config, dict)

        self.config = self.config_setup(config)
        self.process_config = self.process_config_setup(process_config)

    def config_setup(self, config):
        """Setup function for config file. Used to process/add missing information.

        args:
            config (dict): AVMP configuration file

        return (dict): Modified python config dictionary
        """

        return config

    def process_config_setup(self, process_config):
        """Setup function for process config file. Used to process/add missing information.

        args:
            config (dict): Process configuration file

        return (dict): Modified python process config dictionary
        """

        if "allow_ticket_duplication" not in process_config:
            process_config['allow_ticket_duplication'] = False

        return process_config

    def tenAPIcon(self):
        """Check for credentials in config and connect to Tenable IO.
        """
        if 'access_key' in self.config['creds']['tenable'] and 'secret_key' in self.config['creds']['tenable']:

            self.tenAPI = TenableToolsAPI(self.config['creds']['tenable']['access_key'],
                                          self.config['creds']['tenable']['secret_key'])
        else:
            message = 'Tenable access_key and secret_key are required and must be provided in the config file.'
            logging.critical(message)
            raise MissingConfiguration(message)

    def jiraAPIcon(self):
        """Check for credentials in config and connect to Jira.
        """

        if 'server' in self.config['creds']['jira'] and 'username' in self.config['creds']['jira'] and 'password' in self.config['creds']['jira']:

            self.jiraAPI = JiraToolsAPI(self.config['creds']['jira']['server'],
                                        username=self.config['creds']['jira']['username'],
                                        password=self.config['creds']['jira']['password'])
        else:
            message = 'Jira server, username and password are required and must be provided in the config file.'
            logging.critical(message)
            raise MissingConfiguration(message)


if __name__ == '__main__':
    import json
    config_location = 'avmp/test_data/config.json'
    config = json.load(open(config_location, 'r'))

    app = App(config).jiraAPI()



"""Tools for accessing and importing Tenable data.
"""
__copyright__ = "Copyright (C) 2020-2021  Matt Ferreira"
__license__ = "Apache License"

import os
import csv
from datetime import datetime
import logging
from tenable.io import TenableIO


class DataError(Exception):
    """raised error"""


class TenableToolsAPI:
    def __init__(self, access_key, secret_key):
        """Capture access and secret key for Tenable.IO API

        args:
            access_key (str): Access key for Tenable.IO.
            secret_key (str): Secret key for Tenable.IO.

        Return: None
        """
        self.access_key = access_key
        self.secret_key = secret_key

        self.tio = TenableIO(self.access_key, self.secret_key)
        self.scans = self.get_all_scan_names()
        logging.info("Authenticated successfully with Tenable")

    def get_all_scan_names(self):
        scan_list = self.tio.scans.list()

        scans = []
        for scan in scan_list:
            scans.append(scan['name'])

        return scans

    def get_scan_meta_data(self, scan_name):
        """Gets scan meta data for a provided scan_name.

        args:
            scan_name (str): Name of scan found in Tenable.

        Return (dict): Scan meta data.
        """
        # Get list of all scans in Tenable
        scan_list = self.tio.scans.list()

        for scan in scan_list:
            if scan['name'] == scan_name:
                return scan

    def get_scan_info(self, scan_name):
        """Gets scan info for a provided scan_name.

        args:
            scan_name (str): Name of scan found in Tenable.

        Return (dict): Scan meta data.
        """
        # Get list of all scans in Tenable
        scan_list = self.tio.scans.list()

        for scan in scan_list:
            if scan['name'] == scan_name:
                latest_scan_uuid = next(self.tio.scans.history(scan['id']))[
                    'scan_uuid']
                return self.tio.scans.info(scan['id'], latest_scan_uuid)

    def export_latest_scan(self, scan_name, folder, overwrite=None, export_format='csv'):
        """Export the latest scan given a scan name and folder filepath.

        args:
            scan_name (str): Name of scan found in Tenable.
            folder (str): Root folder where scan data should be saved.

        kwargs:
            overwrite (bool): [default: None] Programatic way to determine if existing files should be overwriten.
            export_format (str): [default: csv] Format data should be saved as (this aligns with the TenableIO documentation).

        Return (str): Filepath of scan
        """
        assert isinstance(scan_name, str)
        assert isinstance(folder, str)
        assert isinstance(export_format, str)

        SCAN_NAME = scan_name
        SAVE_FOLDER = os.path.join(folder, SCAN_NAME)

        # Creates sub-folder if one does not exist
        if os.path.isdir(SAVE_FOLDER) is not True:
            logging.info('subFolder not found... Creating now.')
            os.makedirs(SAVE_FOLDER)

        scan_meta_data = self.get_scan_meta_data(SCAN_NAME)

        # Get latest scan history id
        history_id, scan_date = self._get_latest_history_id(
            str(scan_meta_data['id']))
        scan_date = datetime.fromtimestamp(scan_date).strftime('%Y-%m-%d')

        filename = f'{scan_date} - {SCAN_NAME}'
        filepath = f'{SAVE_FOLDER}/{filename}.{export_format}'

        # Check if file exists
        if os.path.exists(f'{filepath}'):
            if overwrite == False or overwrite == True:
                pass
            else:
                user_input = input(
                    f'"{filename}" exists, would you like to overwrite it? (n): ')
                if user_input.lower() == 'y' or user_input.lower() == 'yes':
                    overwrite = True
                else:
                    overwrite = False
        else:
            overwrite = True

        # Checking if overwriting is permitted
        if overwrite != False:
            # Save to supplied folder
            logging.info('Downloading scan to supplied folder')
            with open(filepath, 'wb') as openFile:
                self.tio.scans.export(
                    scan_meta_data['id'], history_id=history_id, fobj=openFile, format=export_format)
        else:
            logging.info('Existing file found, overwriting not permitted.')

        return filepath

    def _get_latest_history_id(self, scan_id):
        """Class function to identify latest history id.

        args:
            scan_id (str): Scan id to capture latest history id.

        Return: history_id (str), scan_date (datetime).
        """
        assert isinstance(scan_id, str)

        first = True
        for history in self.tio.scans.history(scan_id):

            history_time_start = history['time_start']

            if first:
                scan_date = history_time_start
                history_id = history['id']
                first = False
            else:
                if scan_date < history_time_start:
                    scan_date = history_time_start
                    history_id = history['id']

        return history_id, scan_date

    def check_scan_in_progress(self, scan_name):
        scan_meta_data = self.get_scan_info(scan_name)

        if scan_meta_data['status'] == 'running':
            return True
        else:
            return False


class TenableToolsCSV:
    def __init__(self,  filepath, min_cvss_score=None):
        assert isinstance(filepath, str)

        self.filepath = filepath
        self.min_cvss_score = min_cvss_score
        self.columns, self.rows = self._importer()

    def _importer(self):
        """Import Tenable csv file.

        Return: column names (tuple), rows (tuple)
                returns 'None' if no data is available.
        """
        with open(self.filepath, 'r', encoding='utf-8-sig') as import_file:
            reader = csv.reader(import_file)
            first = True
            final_list = []
            for num, row in enumerate(reader):
                temp_dict = {}
                if first == True:
                    title_info = (tuple(row))
                    first = False
                else:
                    for sub_num, item in enumerate(row):
                        temp_dict[title_info[sub_num]] = item
                    if self.min_cvss_score != None:
                        if float(temp_dict['CVSS']) >= float(self.min_cvss_score):
                            final_list.append(temp_dict)

        if len(final_list) == 0:
            return None

        return tuple(title_info), tuple(final_list)

    def group_by(self, column_name):
        """Group data by a given column name.

        args:
            column_name (str): Column name to group by.

        return (dict): Grouped dictionary
                       return 'None' if no data to group.
        """

        # Check if data was imported
        if len(self.rows) != 0:
            data = self.rows

            # Check if column name exists
            column_exists = False
            for row in data:
                for key in row.keys():
                    if key == column_name:
                        column_exists = True
            # Error out if column does not exist
            if column_exists == False:
                raise DataError('group_by column not found.')

            # Group by column name
            grouped_dict = {}
            for row in data:
                if row[column_name] not in grouped_dict:
                    grouped_dict[row[column_name]] = {}

                next_num = len(grouped_dict[row[column_name]]) + 1

                grouped_dict[row[column_name]][next_num] = row

            return grouped_dict
        else:
            return None

    @staticmethod
    def organize(grouped_dict):
        """Given data from grouped_dict function seperate the
        vulnerability and host information.

        args:
            grouped_dict (dict): Data from grouped_dict function.

        return (dict): Organized dictionary [i.e. {'Vuln Data': {...}, 'Hosts': {...}]

        """
        static_data_keys = ['CVE',
                            'CVSS',
                            'CVSS Base Score',
                            'CVSS Temporal Score',
                            'CVSS Temporal Vector',
                            'CVSS Vector',
                            'CVSS3 Base Score',
                            'CVSS3 Temporal Score',
                            'CVSS3 Temporal Vector',
                            'CVSS3 Vector',
                            'Description',
                            'Name',
                            'Plugin Family',
                            'Plugin ID',
                            'Risk',
                            'Solution',
                            'See Also',
                            'Synopsis']

        organized_data = {}

        # Loop through grouping
        for group_name, group_value in grouped_dict.items():
            new_grouped_dict = {}
            static_data = {}
            host_data = {}

            first = True
            # Loop through items
            for key, item in group_value.items():
                temp_host_items = {}

                # Loop through value pairs
                for title, value in item.items():

                    # Extract vulnerability information
                    if first and title in static_data_keys:
                        static_data[title] = value

                    # Extract host information
                    elif title not in static_data_keys:
                        temp_host_items[title] = value

                if first:
                    new_grouped_dict['Vuln Data'] = static_data
                    first = False
                host_data[key] = temp_host_items

                new_grouped_dict['Hosts'] = host_data
            organized_data[group_name] = new_grouped_dict

        return organized_data


def build_jira_description(ticket):
    """Given a ticket after running TenableToolsCSV.organize this will build a formatted jira description.

    args:
        ticket (dict): This must br from TenableToolsCSV.organize

    return (str): Description string.
    """
    jira_description = 'h3. (!) *PLEASE BE SURE TO PROVIDE EVIDENCE!* *Screenshots, a config output, etc.* (!)\n'
    jira_description += '----\n'
    jira_description += 'h2. *{}* '.format(ticket['Vuln Data']['Name'].strip())
    jira_description += '*({})*\n'.format(
        ticket['Vuln Data']['Plugin ID'].strip())
    jira_description += '{panel:title=Overview}\n'
    jira_description += 'h3. +Synopsis+\n'
    jira_description += ticket['Vuln Data']['Synopsis'].strip() + '\n'
    jira_description += 'h3. +Description+\n'
    jira_description += ticket['Vuln Data']['Description'].strip() + '\n'
    jira_description += 'h3. +Solution+\n'
    jira_description += ticket['Vuln Data']['Solution'].strip() + '\n'
    jira_description += 'h3. +See Also+\n'
    jira_description += ticket['Vuln Data']['See Also'].strip() + '\n'
    jira_description += '{panel}\n'
    jira_description += '{panel:title=Supplement Information}\n'

    # Loop through static data
    for key, value in ticket['Vuln Data'].items():
        if key not in ['Name', 'Synopsis', 'Description', 'Solution', 'See Also']:
            jira_description += '* *{}:* {}\n'.format(key, value)

    # Build host section and table header
    jira_description += '{panel}\n'
    jira_description += '{panel:title=Hosts}\n'
    jira_description += '||IP Address||Port||Host||OS||MAC Address||Scan Start||Scan End||Plugin Output||\n'

    # Loop through hosts
    for host in ticket['Hosts'].values():
        jira_description += '|'
        jira_description += '{} |'.format(host['IP Address'])
        jira_description += '{} |'.format(host['Port'])
        jira_description += '{} |'.format(host['Host'])
        jira_description += '{} |'.format(host['OS'])
        jira_description += '{} |'.format(host['MAC Address'])
        jira_description += '{} |'.format(zulu_to_mdy(host['Host Start']))
        jira_description += '{} |'.format(zulu_to_mdy(host['Host End']))
        code_str = '{code}'
        jira_description += '{}{}{} |'.format(code_str,
                                              host['Plugin Output'], code_str)
        jira_description += '\n'

    jira_description += '{panel}\n'
    jira_description += '----\n'
    jira_description += 'h3. (!) *PLEASE BE SURE TO PROVIDE EVIDENCE!* *Screenshots, a config output, etc.* (!)\n'

    return jira_description


def zulu_to_mdy(zulu_date):
    """Converts Tenables time format (Zulu) to MDY (Month, Day, Year)

    args:
        zulu_date (str): Tenable given time format

    return (str): MDY (Month, Day, Year)
    """
    return datetime.strptime(zulu_date[:19], "%Y-%m-%dT%H:%M:%S").strftime('%m-%d-%Y %I:%M %p')



"""Functions for setting up logging accross this project.
"""
__copyright__ = "Copyright (C) 2020-2021  Matt Ferreira"
__license__ = "Apache License"

import os
import sys
import json
import logging


def logging_setup(filename, stdout=False):
    """Standard logging setup.

    args:
        filename (str): Script filename.

    kwargs:
        stdout (bool): Switch logging to stdout for testing

    return: None
    """
    isinstance(filename, str)

    if os.path.isdir('logs') == False:
        os.mkdir('logs')
        logging.info('Creating logs folder.')

    # Logging configuration
    if stdout == False:
        fmtstr = '%(asctime)s:%(levelname)s:%(module)s:%(message)s'
        logging.basicConfig(
            filename='logs/{}.log'.format(filename[:-3]),
            level=logging.DEBUG,
            filemode='a',
            format=fmtstr
        )
    else:
        fmtstr = '[%(levelname)s]\t%(message)s'
        logging.basicConfig(
            stream=sys.stdout,
            level=logging.INFO,
            format=fmtstr)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['avmp', 'avmp.core', 'avmp.tools', 'avmp.utils']

package_data = \
{'': ['*']}

install_requires = \
['click==7.1.2',
 'docopt>=0.6.2,<0.7.0',
 'jira==2.0.0',
 'pyTenable==1.2.6',
 'requests==2.25.1']

entry_points = \
{'console_scripts': ['avmp = avmp.core.cli: main']}

setup_kwargs = {
    'name': 'avmp',
    'version': '0.1.1',
    'description': 'Command line vulnerability program manager.',
    'long_description': '![Alt text](https://raw.githubusercontent.com/RackReaver/AVMP/main/logo.png?raw=true "logo")\n\n# Automated Vulnerability Management Program (AVMP)\n\nA collection of tools for managing and automating vulnerability management.\n\nStreamline the way vulnerability management programs are created and run. This project is made to be modular so automation can be put into place at any program level.\n\n---\n\n## Table of Contents\n\n- [Things to Note](#things-to-note)\n- [Intallation](#installation)\n- [Running the tests](#running-the-tests)\n- [Deployment](#deployment)\n- [How to use](#how-to-use)\n  - [Folder Structure](#folder-structure-vulnmanager)\n  - [Main configuration file "config.json"](#main-configuration-file-configjson)\n  - [Dynamic process config "dynamic_process_config.json](#dynamic-process-config-dynamicprocessconfigjson)\n  - [Static process config "static_process_config.json"](#static-process-config-staticprocessconfigjson)\n- [TO-DO](#to-do)\n- [Authors](#authors)\n- [License](#license)\n\n---\n\n## Things to Note\n\n1. **API keys for both Tenable IO and Jira are required.**\n\n2. I developed this tool using Jira on-prem and assume it would work for the cloud version as well however am unable to verify that.\n\n3. There is a bit of setup to use the tool as it is in development, I am planning on creating a quick start script but until then please see [How to use](#how-to-use).\n\n## Installation\n\n```\npip install git+https://github.com/RackReaver/AVMP\n```\n\n## Running the tests\n\nCheck code coverage\n\n```\n>>> coverage run -m pytest\n>>>\n>>> coverage report\n```\n\n## Deployment\n\nAt this time the tool can only be deployed locally.\n\n## How to use\n\n```\n$ avmp --help\n      __      ____  __ _____\n     /\\ \\    / /  \\/  |  __ \\\n    /  \\ \\  / /| \\  / | |__) |\n   / /\\ \\ \\/ / | |\\/| |  ___/\n  / ____ \\  /  | |  | | |\n /_/    \\_\\/   |_|  |_|_|\n\nA collection of tools for managing and automating vulnerability management.\n\nUsage:\n    avmp run [--config filepath] <process_config>\n    avmp update [--config filepath] <ticket_db_filepath>\n    avmp -h | --help\n    avmp --version\n\nOptions:\n    -h --help           Show this screen.\n    --version           Show version.\n    --config=filepath   AVMP configuration file [default: config.json]\n```\n\n#### Folder Structure `vuln_manager`:\n\n```\nvuln_manager\n|\n+-- process_configs\n|  |\n|  +-- dynamic/     # Configurations for generating vulnerability tickets\n|  +-- static/      # Configurations for generating repetitive project/task tickets\n|\n+-- config.json\n+-- tickets.db      # This is generated automatically and is mapped to in the process_configs\n```\n\n#### Main configuration file `config.json`:\n\n```json\n{\n    "creds": {\n        "tenable": {\n            "access_key": "",\n            "secret_key": ""\n        },\n        "jira": {\n            "server": "",\n            "username": "",\n            "password": ""\n        }\n    },\n    "types": {\n        "JIRA_PROJECT_ID": ["JIRA_FIELD_1", "JIRA_FIELD_2", "JIRA_FIELD_3"]\n    },\n    "due_dates": {\n        "Critical": "DAYS_TO_PATCH",\n        "High": "DAYS_TO_PATCH",\n        "Medium": "DAYS_TO_PATCH",\n        "Low": "DAYS_TO_PATCH",\n    },\n    "priorities": {\n        "Critical": "JIRA_ID",\n        "High": "JIRA_ID",\n        "Medium": "JIRA_ID",\n        "Low": "JIRA_ID"\n}\n```\n\n| Field      | Required | Description                                                                                                                                                          |\n| ---------- | -------- | -------------------------------------------------------------------------------------------------------------------------------------------------------------------- |\n| creds      | yes      | Data required from APIs to run package                                                                                                                               |\n| types      | no       | List of required fields for a given Jira project (not required, but a good idea to ensure process_configs contain all required fields before making an API request). |\n| due_dates  | yes      | Used to set Jira due date based on Tenable\'s severity rating.                                                                                                        |\n| priorities | yes      | Mapping Tenable severity rating to Jira priorities (defaults to `Low` if others are unavailable).                                                                    |\n\n#### Dynamic process config `dynamic_process_config.json`:\n\nSee [examples](examples/dynamic_process_configs) for context on use cases.\n\n```json\n{\n  "process_type": "dynamic",\n  "allow_ticket_duplication": "False",\n  "scan_name": "TENABLE_SCAN_NAME",\n  "max_tickets": 10,\n  "assignee": "",\n  "min_cvss_score": 6.0,\n  "ticket_db_filepath": "tickets.db",\n  "default_ticket_status": "Open",\n  "time_saved_per_ticket": "10m",\n  "root_ticket": "",\n  "comments": [],\n  "data": {\n    "project": { "key": "JIRA_PROJECT_KEY" },\n    "summary": "Vuln: ",\n    "description": "",\n    "issuetype": { "id": "JIRA_ISSUE_TYPE_ID" },\n    "priority": { "id": "" },\n    "duedate": ""\n  }\n}\n```\n\n| Field                    | Required | Description                                                                                                                        |\n| ------------------------ | -------- | ---------------------------------------------------------------------------------------------------------------------------------- |\n| process_type             | yes      | Must be `dynamic`                                                                                                                  |\n| allow_ticket_duplication | no       | [default: false] Prevent multiple tickets for same plugin_id to be generated (This is based on the `ticket_db_filepath` provided). |\n| scan_name                | yes      | Name of scan inside of Tenable IO                                                                                                  |\n| max_tickets              | no       | Number of tickets to be created each time this configuration is used (optional - will create all if value is blank).               |\n| assignee                 | no       | Username to assign all created tickets to (optional).                                                                              |\n| min_cvss_score           | yes      | This is based on the CVSS Base Score provided by Tenable IO, allows for configurations based on severity.                          |\n| ticket_db_filepath       | yes      | Location of SQLite database file for tracking tickets (DB will be created if ones doesn\'t already exist on the path provided).     |\n| default_ticket_status    | yes      | First status for database entry, this will change when the auto updater is run.                                                    |\n| time_saved_per_ticket    | yes      | Jira time value to log work for calculating time saved.                                                                            |\n| root_ticket              | no       | If unable to log work against newly created ticket this value will provide a ticket that allows work to be logged against it.      |\n| comments                 | no       | A list of strings that will generate comments.                                                                                     |\n| data                     | yes      | API values required to generate a Jira ticket (issue).                                                                             |\n\n#### Static process config `static_process_config.json`:\n\nSee [examples](examples/static_process_configs) for context on use cases.\n\n```json\n{\n  "process_type": "static",\n  "time_saved_per_ticket": "5m",\n  "time_saved_comment": "Time saved through automation",\n  "parent_ticket": {\n    "project": { "key": "JIRA_PROJECT_KEY" },\n    "summary": "SUMMARY",\n    "description": "DESCRIPTION",\n    "issuetype": { "name": "ISSUE_TYPE_NAME" },\n    "assignee": { "name": "" },\n    "priority": { "id": "PRIORITY_ID" }\n  },\n  "sub_tasks": {\n    "sub_task_1": {\n      "project": { "key": "JIRA_PROJECT_KEY" },\n      "summary": "SUMMARY",\n      "description": "DESCRIPTION",\n      "issuetype": { "name": "Sub-task" },\n      "assignee": { "name": "" }\n    },\n    "sub_task_2": {\n      "project": { "key": "JIRA_PROJECT_KEY" },\n      "summary": "SUMMARY",\n      "description": "DESCRIPTION",\n      "issuetype": { "name": "Sub-task" },\n      "assignee": { "name": "" }\n    }\n  }\n}\n```\n\n| Field Name            | Required | Description                                                                  |\n| --------------------- | -------- | ---------------------------------------------------------------------------- |\n| process_name          | yes      | Must be `static`                                                             |\n| time_saved_per_ticket | no       | Jira time value to log work for calculating time saved.                      |\n| time_saved_comment    | no       | Comment for Jira work log for time saved.                                    |\n| parent_ticket         | yes      | API values required to generate a Jira ticket (issue).                       |\n| sub_tasks             | no       | JSON container for any sub tasks that should be created under parent ticket. |\n\n## TO-DO\n\n- Add persistent logging to wrapper.main() as a return value\n- Add ticket reference table to database\n  - Track project and process ticket numbers for automated linking\n- Add support for [SecurityScorecard](https://securityscorecard.com/)\n- Build tests for code base\n- [x]Add example dynamic and static process configs\n- Add database tracking of static_process tickets\n\n## Authors\n\n- **Matt Ferreira** - _Developer_ - [RackReaver](https://github.com/RackReaver)\n\n## License\n\nThis project is licensed under the Apache License - see the [LICENSE](LICENSE) file for details\n',
    'author': 'RackReaver',
    'author_email': 'rackreaver@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/RackReaver/AVMP',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""Python tools for package.
"""
__copyright__ = "Copyright (C) 2020-2021  Matt Ferreira"
__license__ = "Apache License"

import json
import requests


class SlackTools:
    def __init__(self):
        pass

    @staticmethod
    def push_alert(webhook_url, message, channel=None, custom_payload=None):
        """Push an alert to slack channel given a webhooks url and message.

        args:
            webhook_url (str): Incoming Webhook provided by slack.
            message (str): Message to push via webhook.
        kwargs:
            channel (str): Allow messages to be sent to any public channel.
            custom_payload (json): Overwrites message and channel inputs.
        """
        if custom_payload == None:
            if channel != None:
                payload = {"channel": channel,
                           "blocks": [
                               {
                                   "type": "section",
                                   "text": {
                                       "type": "mrkdwn",
                                       "text": message
                                   }
                               }
                           ]
                           }
            else:
                payload = {
                    "blocks": [
                        {
                            "type": "section",
                            "text": {
                                "type": "mrkdwn",
                                        "text": message
                            }
                        }
                    ]
                }

        r = requests.post(webhook_url,
                          data=json.dumps(payload),
                          headers={'Content-Type': 'application/json'})

        return r
